@@ -6,6 +6,7 @@ pybufrkit.decoder
 from __future__ import absolute_import
 from __future__ import print_function
 
+import sys
 import functools
 import logging
 # noinspection PyUnresolvedReferences
@@ -18,13 +19,18 @@ from pybufrkit.constants import (BITPOS_START,
                                  NUMERIC_MISSING_VALUES,
                                  PARAMETER_TYPE_TEMPLATE_DATA,
                                  PARAMETER_TYPE_UNEXPANDED_DESCRIPTORS)
+from pybufrkit.errors import PyBufrKitError
 from pybufrkit.bitops import get_bit_reader
 from pybufrkit.bufr import BufrMessage
+from pybufrkit.tables import TableGroupCacheManager
 from pybufrkit.templatedata import TemplateData
 from pybufrkit.coder import Coder, CoderState
+from pybufrkit.dataprocessor import BufrTableDefinitionProcessor
 from pybufrkit.templatecompiler import CompiledTemplateManager, process_compiled_template
+from pybufrkit.script import ScriptRunner
+
+__all__ = ['Decoder', 'generate_bufr_message']
 
-__all__ = ['Decoder']
 
 log = logging.getLogger(__file__)
 
@@ -71,7 +77,7 @@ class Decoder(Coder):
         """
         idx = s.find(start_signature) if start_signature is not None else 0
         if idx == -1:
-            raise RuntimeError('Cannot find start signature: {}'.format(start_signature))
+            raise PyBufrKitError('Cannot find start signature: {}'.format(start_signature))
         s = s[idx:]
 
         bit_reader = get_bit_reader(s)
@@ -147,7 +153,8 @@ class Decoder(Coder):
                 log.debug('Skipping {} bits to end of the section'.format(nbits_unread))
                 bit_reader.read_bin(nbits_unread)
             elif nbits_unread < 0:
-                raise RuntimeError('Read exceeds declared section length: {}'.format(section.section_length.value))
+                raise PyBufrKitError('Read exceeds declared section {} length: {} by {} bits'.format(
+                    section.get_metadata('index'), section.section_length.value, -nbits_unread))
 
         return bit_reader.get_pos() - section.get_metadata(BITPOS_START)
 
@@ -289,10 +296,8 @@ class Decoder(Coder):
         min_value = bit_reader.read_bytes(nbytes_min_value)
         nbits_diff = bit_reader.read_uint(NBITS_FOR_NBITS_DIFF)
 
-        if nbits_diff:  # non-zero nbits_diff
-            assert min_value == b'\0' * nbytes_min_value, (
-                '{}: Different string must be compressed'
-                ' with empty min value'.format(descriptor))
+        if min_value in (b'\0' * nbytes_min_value or b'\xff' * nbytes_min_value):
+            min_value = b''
 
         # special cases: all missing or all equals
         if min_value is None or nbits_diff == 0:
@@ -302,7 +307,8 @@ class Decoder(Coder):
                 decoded_values.append(min_value)
         else:
             for decoded_values in state.decoded_values_all_subsets:
-                decoded_values.append(bit_reader.read_bytes(nbits_diff))
+                diff_value = bit_reader.read_bytes(nbits_diff)
+                decoded_values.append(min_value + diff_value)
 
     def process_codeflag(self, state, bit_reader, descriptor, nbits):
         (self.process_codeflag_compressed if state.is_compressed else
@@ -382,7 +388,11 @@ class Decoder(Coder):
             decoded_values.append(value)
 
 
-def generate_bufr_message(decoder, s, info_only=False, *args, **kwargs):
+DATA_CATEGORY_DEFINE_BUFR_TABLES = 11
+
+
+def generate_bufr_message(decoder, s, info_only=False, continue_on_error=False, filter_expr=None,
+                          *args, **kwargs):
     """
     This is a generator function that processes the given string for one
     or more BufrMessage till it is exhausted.
@@ -391,16 +401,51 @@ def generate_bufr_message(decoder, s, info_only=False, *args, **kwargs):
     :param bytes s: String to decode for messages
     :return: BufrMessage object
     """
+    sr = ScriptRunner(filter_expr, mode='eval') if filter_expr is not None else None
     idx_start = 0
     while idx_start < len(s):
         idx_start = s.find(MESSAGE_START_SIGNATURE, idx_start)
         if idx_start < 0:
-            raise StopIteration
-        bufr_message = decoder.process(
-            s[idx_start:], start_signature=None, info_only=info_only, *args, **kwargs
-        )
-        # If data section is not decoded, we rely on the declared length for the message length
-        if info_only:
-            bufr_message.serialized_bytes = s[idx_start: idx_start + bufr_message.length.value]
-        idx_start += len(bufr_message.serialized_bytes)
-        yield bufr_message
+            return
+        try:
+            matched = True
+            if filter_expr:
+                bufr_message = decoder.process(
+                    s[idx_start:], start_signature=None, info_only=True, *args, **kwargs
+                )
+                matched = sr.run(bufr_message)
+                if matched and not info_only:
+                    bufr_message = decoder.process(
+                        s[idx_start:], start_signature=None, info_only=False, *args, **kwargs
+                    )
+            else:
+                bufr_message = decoder.process(
+                    s[idx_start:], start_signature=None, info_only=info_only, *args, **kwargs
+                )
+            # If data section is not decoded, we rely on the declared length for the message length
+            if info_only:
+                bufr_message.serialized_bytes = s[idx_start: idx_start + bufr_message.length.value]
+            else:
+                if (bufr_message.data_category.value == DATA_CATEGORY_DEFINE_BUFR_TABLES
+                        and bufr_message.n_subsets.value > 0):
+                    _, b_entries, d_entries = BufrTableDefinitionProcessor().process(bufr_message)
+                    TableGroupCacheManager.invalidate()
+                    TableGroupCacheManager.add_extra_entries(b_entries, d_entries)
+            idx_start += len(bufr_message.serialized_bytes)
+
+            if matched:
+                yield bufr_message
+
+        except PyBufrKitError as e:
+            if not continue_on_error:
+                raise e
+            print('Continuing on next message and ignoring error: {}'.format(e), file=sys.stderr)
+            if info_only:
+                idx_start += 1
+            else:
+                try:
+                    bufr_message = decoder.process(
+                        s[idx_start:], start_signature=None, info_only=True, *args, **kwargs)
+                    idx_start += bufr_message.length.value
+                except PyBufrKitError as e:
+                    idx_start += 1
